#!/usr/bin/perl
# -----------------------------------------------------------------------------
#   ident "@(#)addon.pl 1.17 04/01/30 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------
package ADDON;

use strict;
use variables;


#
# What and when ?
#
#
# Packages and patches installed during a call to this script depend on:
# . the phase assigned to the package or the patch.
# . the scope of the package
# . the current phase
#
# Herefafter actions performed

#
# When ?
#   depends on the current phase and the required phase
#
#    current phase                     !        phase to install
#                                      !   on MEN    !   on diskless
# -------------------------------------------------------------------------
#   I_MEN (during Solaris on MEN)      !     I       !       -
#   S_MEN (after Solaris on MEN)       !     S       !       -
#   F_MEN (after FS on MEN)            !     F       !       -
#   I_NMEN (after Solaris on diskless) !     -       !       I   if men #1
#          first step)                 !             !
#   S_NMEN (after Solaris on diskless) !     -       !       S   if men #1
#          second step)                !             !
#   F_NMEN (after FS on diskless)      !     -       !       F   if men #1

# 
# Where ?
#   depends on the scope
#
# Scope LOCAL: on / for MEN and diskless (root file systtem for each diskless)
#
# Scope USR_SPECIFIC or USR_SOLARIS: =LOCAL for MEN and on /export/os.. 
#           for diskless (only for men #1)
#
# Scope SHARED: only for men #1: on the shared directory (not for S_MEN: not
#               already created).
#


my $FOR_MEN=$FALSE;
my $FOR_NMEN=$FALSE;
my $WHERE="";

my $SCOPE;        # to_be_processed can change the scope defined by the user
my $CHANGE;       # indicate that a change has been made and a 
                  # warning must be displayed
my $DIR;          # directory where the package/patch is really located
my $SUBDIR;       # subdirectory where the package/patch is really located
my $MANAGE_SHARE; # must share/unshare command be executed
my $INSTALL_METHOD; # Installation method

#-----------------------------------------------------------
#
#   used for testing (retrieve local module variables)
#
#-----------------------------------------------------------

sub get_info {
  my ($for_men, $for_nmen, $where, $scope, $change, $method) = @_;

  $$for_men = $FOR_MEN;
  $$for_nmen = $FOR_NMEN;
  $$where = $WHERE;
  $$scope = $SCOPE;
  $$change = $CHANGE;
  $$method = $INSTALL_METHOD;
}

#-----------------------------------------------------------
#
#   determine if a element is a duplicate
#   possible when SMOSSERVICE is used with a addon.conf
#   file designed for patchadd (upward compatibility)
#   do not consider the patch as duplicate if
#   the installation method is different
#
#-----------------------------------------------------------

sub is_a_duplicate {

  my ($idx, $node_type, $arraypar) = @_;

  # check if the element is a duplicate
  # check all elements before the considered element
  # -> duplicate if already defined for the type of node

  my @array= @$arraypar;

  for (my $i = 0 ; $i < $idx ; $i++) {
    if (($array[$i]->{reference} eq $array[$idx]->{reference})
	&& ($array[$i]->{method} eq $array[$idx]->{method})) {
      if ($array[$i]->{$node_type}) {
	# the same reference also installs for the same type of node
	# with the same method
	return $TRUE;
      }
    }
  }

  return $FALSE;
}

#-----------------------------------------------------------
#
#   determine if the patch must be installed
#
#-----------------------------------------------------------

sub to_be_processed {

  my ($idx) = @_;

  my $type  = $ADDON_LIST[$idx]{type};
  my $phase = $ADDON_LIST[$idx]{phase};
  my $men   = $ADDON_LIST[$idx]{men};
  my $nmen  = $ADDON_LIST[$idx]{nmen};
  my $scope = $ADDON_LIST[$idx]{scope};
  my $dir = $ADDON_LIST[$idx]{dir};
  my $subdir = $ADDON_LIST[$idx]{subdir};
  my $method =$ADDON_LIST[$idx]{method};
  
  
  $SCOPE = $scope;
  $CHANGE = $FALSE;
  
  $FOR_MEN = $FALSE ;
  $FOR_NMEN = $FALSE ;
  $INSTALL_METHOD = $method;
  
  if (($type ne "PACKAGE") && ($type ne "PATCH")) {
    COMMON::error("Internal error: to_be_processed: invalid type $type");
  }
  

  $WHERE = "";
  
  my $installed = $FALSE;
  
  # determine if it's a good phase for MEN
  my $good_phase_for_men = $FALSE;
  if ( 
      (($phase eq "I") && ($PHASE eq "I_MEN"))
      || ($phase eq "S") && ($PHASE eq "S_MEN")
      || (($phase eq "F") && ($PHASE eq "F_MEN"))
     ) {
    $good_phase_for_men = $TRUE;
    $INSTALL_METHOD = $STANDARD_METHOD;
  }
  
  # determine if it's a good phase for diskless
  my $good_phase_for_diskless = $FALSE;
  if (
      (($phase eq "I") && ($PHASE eq "I_NMEN")) 
      || ($phase eq "S") && ($PHASE eq "S_NMEN") 
      || (($phase eq "F") && ($PHASE eq "F_NMEN"))
     ) {
    $good_phase_for_diskless = $TRUE;
  }
  
  # scope: SHARED
  if ($scope eq "SHARED") {
    if (($good_phase_for_men) && ($PHASE ne "I_MEN") 
	&& ($PHASE ne "S_MEN") && ($NODE_ID eq $MEN1_ID)) {
      $FOR_MEN = $TRUE;
      return $TRUE ;
    } 
    return $FALSE;
  }
  
  # determine the directory to be used depending on the impacted node type
  $DIR = $dir;
  $SUBDIR = $subdir;
  $MANAGE_SHARE = $TRUE;
  
  if ($dir eq "NHAS") {
    $MANAGE_SHARE = $FALSE;
    $DIR = $NHAS_DIR;
    if ($subdir eq "") {
      if ($type eq "PACKAGE") {
	if ($good_phase_for_men) {
	  $SUBDIR = $MEN_NHAS_PACKAGE_SUBDIR;
	} else {
	  $SUBDIR = $DISKLESS_NHAS_PACKAGE_SUBDIR;
	}
      } else {
	if ($good_phase_for_men) {
	  $SUBDIR = $MEN_NHAS_PATCH_SUBDIR;
	} else {
	  $SUBDIR = $DISKLESS_NHAS_PATCH_SUBDIR;
	}
      }
    }
  } elsif ($dir eq "SOLARIS") {
    $MANAGE_SHARE = $FALSE;
    # distrib depends on the phase
    if ($good_phase_for_men) {
      $DIR = $MEN_SOLARIS_DIR;
    } else {
      $DIR = $DISKLESS_SOLARIS_DIR;
    }
    if ($subdir eq "") {
      if ($type eq "PACKAGE") {
	if ($good_phase_for_men) {
	  $SUBDIR = $MEN_SOLARIS_PACKAGE_SUBDIR;
	} else {
	  $SUBDIR = $DISKLESS_SOLARIS_PACKAGE_SUBDIR;
	}
      } else {
	if ($good_phase_for_men) {
	  $SUBDIR = $MEN_SOLARIS_PATCH_SUBDIR;
	} else {
	  $SUBDIR = $DISKLESS_SOLARIS_PATCH_SUBDIR;
	}
      }
    }
  } elsif (($dir eq $MEN_SOLARIS_DIR) 
	   || ($dir eq $DISKLESS_SOLARIS_DIR)
	   || ($dir eq $NHAS_DIR)) {
    $MANAGE_SHARE = $FALSE;
  }
  

  #
  # scope: component installed once for all diskless
  # (in particular not re-installed when adding a diskless
  #
  if (($scope eq "USR_SPECIFIC") 
      || ($scope eq "USR_SOLARIS")
      || ($scope eq "CLONE_OPT")) {
    
    # MASTER-ELIGIBLE NODES
    
    if (($good_phase_for_men) && ($men)) {
      
      $FOR_MEN = $TRUE;
      # in that case, the scope is changed to LOCAL 
      # (/usr is on root file system)
      $SCOPE = "LOCAL";
      # the warning is removed
      #	$CHANGE = $TRUE;
      $installed = $TRUE ;
    } 
    
    # DISKLESS
    
    # to be installed only at the first installation and not when adding a
    # diskless
    if (($good_phase_for_diskless) && ($nmen) && ($NODE_ID eq $MEN1_ID) 
	&& ($INSTALL_TYPE eq "CLUSTER")) {
      # be aware that the install_men_xxx is called (historical reason ...)
      $WHERE = "SOLARIS";
      $FOR_NMEN = $TRUE;
      $installed = $TRUE;
      
      # if it's a Solaris Patch
      # determine if we can use smosservice
      if (($type eq "PATCH") 
	  && ($INSTALL_METHOD == $SMOSSERVICE_METHOD)
	  && (($PHASE eq "I_NMEN") || ($PHASE eq "S_NMEN"))) {
	# if the patch is a T-patch, we cannot use smosservice
	if (substr($ADDON_LIST[$idx]{reference},0,1) eq "T") {
	  $INSTALL_METHOD=$STANDARD_METHOD;
	}
	if (is_a_duplicate($idx, "nmen", \@ADDON_LIST)) {
	  # cancel the installation
	  $installed = $FALSE;
	}
      }
    }

    return $installed;
    
  } else {
    
    #
    # scope: LOCAL (test for both MEN and NMEN)
    #
    
    # MASTER-ELIGIBLE NODES
    
    if (($good_phase_for_men) && ($men)) {
      $FOR_MEN = $TRUE ;
      $installed = $TRUE;
    }
    
    #
    # DISKLESS
    #
    if (($good_phase_for_diskless) && ($nmen) && ($NODE_ID eq $MEN1_ID)) {
      
      # install a Solaris patch (phase I_NMEN or S_NMEN) with smosservice
      # only once (during the cluster installation)
      # do not take into account patch with smosservice for a non-Solaris
      # patch (use patchadd)
      
      # remember that good_phase_for_diskless=TRUE means
      # that if phase=S_NMEN or phase=I_NMEN means then the patch
      # is "S" or "I" -> It's a Solaris patch (versus a NHAS patch)
      
      if ($type eq "PACKAGE") {
	
	$WHERE = "DISKLESS" ;
	$FOR_NMEN = $TRUE;
	$installed = $TRUE;
	
      } else {
	if ($INSTALL_METHOD == $SMOSSERVICE_METHOD) {
	  if (($PHASE eq "I_NMEN") || ($PHASE eq "S_NMEN")) {
	    # Solaris patch
	    if ($INSTALL_TYPE eq "CLUSTER") {
	      # if the patch is a T-patch, we cannot use smosservice
	      if (substr($ADDON_LIST[$idx]{reference},0,1) eq "T") {
		$INSTALL_METHOD = $STANDARD_METHOD;
		$WHERE = "DISKLESS" ;
		$FOR_NMEN = $TRUE;
		$installed = $TRUE;
	      } else {
		# with smosservice, the patch can be defined with LOCAL
		# and USR_SPECIFIC (upward compatibility)
		if (! is_a_duplicate($idx, "nmen", \@ADDON_LIST)) {
		  $WHERE = "DISKLESS" ;
		  $FOR_NMEN = $TRUE;
		  $installed = $TRUE ;
		}
		# else it's a duplicate -> not installed with smosservice
	      }
	    }
	    # else Solaris patch when adding a diskless -> not installed
	  } else  {
	    # non-Solaris patch when adding a diskless -> install it
	    # with patchadd
	    $WHERE = "DISKLESS" ;
	    $FOR_NMEN = $TRUE;
	    $installed = $TRUE;
	  }
	} else {
	  # smosservice not requested
	  $WHERE = "DISKLESS" ;
	  $FOR_NMEN = $TRUE;
	  $installed = $TRUE;
	}
      }
    }
  } 
    
  return $installed;
}

#-----------------------------------------------------------
# PACKAGES
#-----------------------------------------------------------

#-----------------------------------------------------------
#
#   installing a package
#
#-----------------------------------------------------------

sub install_package {

  my ($i) = @_;

  if (to_be_processed($i)) {
    if ($CHANGE) {
      COMMON::warning("scope $ADDON_LIST[$i]{scope} changed to $SCOPE for $ADDON_LIST[$i]{reference} when installing on MEN\n");
    }
    
    if ($MANAGE_SHARE) {
      COMMON::share($DIR);
    }
    
    if ($FOR_MEN) {
      COMPONENT::install_men_pkg($NODE_ID, $DIR, $SUBDIR,
				 $ADDON_LIST[$i]{reference}, $SCOPE, "") ;
    } 
    if ($FOR_NMEN) {
      if (($SCOPE eq "USR_SPECIFIC")
	  || ($SCOPE eq "USR_SOLARIS")
	  || ($SCOPE eq "CLONE_OPT")) {
	COMPONENT::install_men_pkg($NODE_ID, $DIR, $SUBDIR,
				   $ADDON_LIST[$i]{reference}, $SCOPE, "") ;
      } else {
	COMPONENT::install_nmen_pkg($NODE_ID, $DIR, $SUBDIR,
				    $ADDON_LIST[$i]{reference}, "") ;
      }
    }
  }
}

#-----------------------------------------------------------
#
#   removing a package
#
#-----------------------------------------------------------

sub remove_package {
  
  my ($i) = @_;
  
  if (to_be_processed($i)) {
    if ($FOR_MEN) {
      COMPONENT::remove_men_pkg($NODE_ID, $ADDON_LIST[$i]{reference}, $SCOPE) ;
    } 
    if ($FOR_NMEN) {
      if (($SCOPE eq "USR_SPECIFIC")
	  || ($SCOPE eq "USR_SOLARIS")
	  || ($SCOPE eq "CLONE_OPT")) {
	COMPONENT::remove_men_pkg($NODE_ID, $ADDON_LIST[$i]{reference}, 
				  $SCOPE) ;
      } else {
	COMPONENT::remove_nmen_pkg($NODE_ID, $ADDON_LIST[$i]{reference}) ;
      }
    }
  }
}

#-----------------------------------------------------------
# PATCHES
#-----------------------------------------------------------

#-----------------------------------------------------------
#
#   installing a patch
#
#-----------------------------------------------------------

sub install_patch {

  my ($i) = @_;
  
  if (to_be_processed($i)) {
    if ($CHANGE) {
      COMMON::warning("scope $ADDON_LIST[$i]{scope} changed to $SCOPE for $ADDON_LIST[$i]{reference} when installing on MEN");
    }
    if ($MANAGE_SHARE) {
      COMMON::share($DIR);
    }
    
    if ($FOR_MEN) {
      COMPONENT::install_men_patch($NODE_ID, $DIR, $SUBDIR,
				   $ADDON_LIST[$i]{reference}, $SCOPE) ;
    } 
    if ($FOR_NMEN) {
      if ($INSTALL_METHOD == $SMOSSERVICE_METHOD) {
	COMPONENT::install_sm_nmen_patch($NODE_ID, $DIR, $SUBDIR,
					 $ADDON_LIST[$i]{reference}, 
					 $WHERE) ;
      } else {
	if ($INSTALL_METHOD == $PATCH_WITH_PKGADD_METHOD) {
	  COMPONENT::install_nmen_patch_pkgadd($NODE_ID, $DIR, $SUBDIR,
					       $ADDON_LIST[$i]{reference}, 
					       $WHERE, $SCOPE) ;
	} else {
	  COMPONENT::install_nmen_patch($NODE_ID, $DIR, $SUBDIR,
					$ADDON_LIST[$i]{reference}, 
					$WHERE) ;
	}
      }
    }
    
  }
}

#-----------------------------------------------------------
#
#   removing a patch
#
#-----------------------------------------------------------

sub remove_patch {

  my ($i) = @_;

  if (to_be_processed($i)) {
    if ($FOR_MEN) {
      COMPONENT::remove_men_patch($NODE_ID, $ADDON_LIST[$i]{reference},
				  $SCOPE) ;
    } 
    if ($FOR_NMEN) {
      if ($INSTALL_METHOD == $SMOSSERVICE_METHOD) {
	COMPONENT::remove_sm_nmen_patch($NODE_ID, 
					$ADDON_LIST[$i]{reference}, 
					$WHERE) ;
      } else {
	if ($INSTALL_METHOD == $PATCH_WITH_PKGADD_METHOD) {
	  COMPONENT::remove_nmen_patch_pkgadd($NODE_ID,
					      $DIR,
					      $SUBDIR,
					      $ADDON_LIST[$i]{reference},
					      $WHERE) ;
	} else {
	  COMPONENT::remove_nmen_patch($NODE_ID, 
				       $ADDON_LIST[$i]{reference}, 
				       $WHERE) ;
	}
      }
    }
  }
}

#-----------------------------------------------------------
#
#   install add-on
#
#-----------------------------------------------------------

sub install_addon {

  for (my $i = 0 ; $i < scalar(@ADDON_LIST) ; $i++) {
    if ($ADDON_LIST[$i]{type} eq "PACKAGE") {
      install_package($i);
    } else {
      install_patch($i);
    }
  }
}

#-----------------------------------------------------------
#
#   remove add-on
#
#-----------------------------------------------------------

sub remove_addon {  

  for (my $i =  scalar(@ADDON_LIST) - 1; $i >= 0; $i--) {
    if ($ADDON_LIST[$i]{type} eq "PACKAGE") {
      remove_package($i);
    } else {
      remove_patch($i);
    }
  }
}


#-----------------------------------------------------------
#
#  general function
#
#-----------------------------------------------------------

sub process_addon {

  ($PHASE, $RECOVERY) = @_;
  
  if (! $DO_ADDON) {
    return;
  }
  
  my $name="for unknown phase";
 SWITCH: {
    if ($PHASE eq "S_MEN") {
      $name = "after Solaris installation on master-eligible nodes" ;
      last SWITCH;
    }
    if ($PHASE eq "I_NMEN") {
      $name = "after Solaris service creation" ;
      last SWITCH;
    }
    if ($PHASE eq "S_NMEN") {
      $name = "after diskless creation" ;
      last SWITCH;
    }
    if ($PHASE eq "F_MEN") {
      $name = "after Foundation Services installation on master-eligible nodes" ;
      last SWITCH;
    }
    if ($PHASE eq "F_NMEN") {
      $name = "after Foundation Services installation on diskless nodes" ;
      last SWITCH;
    }
    if ($PHASE eq "A_MEN") {
      $name = "after Application Services installation on master-eligible nodes" ;
      last SWITCH;
    }
    if ($PHASE eq "A_NMEN") {
      $name = "after Application Services installation on diskless nodes" ;
      last SWITCH;
    }
  }
  
  if ($RECOVERY) {
    COMMON::print_recovery_node("Add-on $name: clean-up") ;
    remove_addon();
    
  } else {
    
    COMMON::print_stage_node("Add-on $name") ;
    install_addon();
  }
}

#-----------------------------------------------------------
#
# functions for each phase
#
#-----------------------------------------------------------

sub men_addon_solaris {
  
  ($NODE_ID) = @_;
  
  process_addon("S_MEN", $FALSE);
}

sub recovery_men_addon_solaris {
  
  ($NODE_ID) = @_;
  
  process_addon("S_MEN", $TRUE);
}

#-----------------------------------------------------------

sub nmen_addon_install {
  
  ($NODE_ID) = @_;
  
  process_addon("I_NMEN", $FALSE);
  COMPONENT::sync_sm_nmen_patch($NODE_ID);
}

sub recovery_nmen_addon_install {
  
  ($NODE_ID) = @_;
  
  process_addon("I_NMEN", $TRUE);
}

#-----------------------------------------------------------

sub nmen_addon_solaris {
  
  ($NODE_ID) = @_;
  
  process_addon("S_NMEN", $FALSE);
  COMPONENT::sync_sm_nmen_patch($NODE_ID);
}

sub recovery_nmen_addon_solaris {
  
  ($NODE_ID) = @_;
  
  process_addon("S_NMEN", $TRUE);
}

#-----------------------------------------------------------

sub men_addon_fs {
  
  ($NODE_ID) = @_;
  
  process_addon("F_MEN", $FALSE);
}

sub recovery_men_addon_fs {
  
  ($NODE_ID) = @_;
  
  process_addon("F_MEN", $TRUE);
}

#-----------------------------------------------------------

sub nmen_addon_fs {
  
  ($NODE_ID) = @_;
  
  process_addon("F_NMEN", $FALSE);
  COMPONENT::sync_sm_nmen_patch($NODE_ID);
}

sub recovery_nmen_addon_fs {
  
  ($NODE_ID) = @_;
  
  process_addon("F_NMEN", $TRUE);
}

#------------------------------------------------------------------------------
#
#  Update the finish.sh file with the creation of the admin file
#
#------------------------------------------------------------------------------

sub update_finish_admin {

	print FINISH  <<"FEO";

# create admin file
$RECHO "mail=" > /a/tmp/admin
$RECHO "instance=overwrite" >> /a/tmp/admin
$RECHO "partial=nocheck" >> /a/tmp/admin
$RECHO "runlevel=quit" >> /a/tmp/admin
$RECHO "idepend=nocheck" >> /a/tmp/admin
$RECHO "rdepend=nocheck" >> /a/tmp/admin
$RECHO "space=quit" >> /a/tmp/admin
$RECHO "setuid=nocheck" >> /a/tmp/admin
$RECHO "conflict=nocheck" >> /a/tmp/admin
$RECHO "action=nocheck" >> /a/tmp/admin
$RECHO "basedir=default" >> /a/tmp/admin

FEO
}

#------------------------------------------------------------------------------
#
#  Update the finish.sh file to install packages and patches during
#  the Solaris installation
#
#------------------------------------------------------------------------------

sub update_finish {

  my ($finish, $mountpoint) = @_;

  my $first = $TRUE;
  my $i;

  if (! $DO_ADDON) {
    return;
  }

  $PHASE = "I_MEN";
  COMMON::print_stage_node("Add-on during Solaris installation");

  open (FINISH, ">>$finish") || 
    COMMON::error("Unable to update $finish file") ;

  
  for (my $i = 0 ; $i < scalar(@ADDON_LIST) ; $i++) {
    
    if (to_be_processed($i)) {
      
      my $reference = $ADDON_LIST[$i]{reference};
      
      # if it's the first element, open the file
      # and create the admin file
      if ($first) {
	update_finish_admin();
	print FINISH "$RMKDIR -p $mountpoint\n";
	$first = $FALSE;
      }
      
      if ($CHANGE) {
	COMMON::warning("scope $ADDON_LIST[$i]{scope} changed to $SCOPE for $reference when installing on MEN\n");
      }
      
      # export the directory on the installation server
      if ($MANAGE_SHARE) {
	COMMON::share($DIR);
      }

      if ($ADDON_LIST[$i]{type} eq "PACKAGE") {
	
	COMMON::print_action("Adding commands for package $reference in finish.sh");
	print FINISH "#\n";
	print FINISH "# package $reference\n";
	print FINISH "#\n";
	print FINISH "$RMOUNT $SERVER_IP:/$DIR $mountpoint\n";
	print FINISH "$RPKGADD -M -d $mountpoint/$SUBDIR -R /a -a /a/tmp/admin $reference\n";
	print FINISH "$RUMOUNT $mountpoint\n";
      } else {
	
	COMMON::print_action("Adding commands for patch $reference in finish.sh");

	print FINISH "#\n";
	print FINISH "# patch $reference\n";
	print FINISH "#\n";
	print FINISH "$RMOUNT $SERVER_IP:/$DIR $mountpoint\n";
	print FINISH "$RPATCHADD -M $mountpoint/$SUBDIR -R /a $reference\n";
	print FINISH "$RUMOUNT $mountpoint\n";
      }
    } # if
  } # for
  
  if (! $first) {
    # at least on element: mounting point has been created
    print FINISH "$RRMDIR $mountpoint\n";
  }
  close(FINISH);
}

{
}
